@@ -1,4 +1,4 @@
-/* $Id: getifaddr.h,v 1.8 2013/03/23 10:46:54 nanard Exp $ */
+/* $Id: getifaddr.h,v 1.10 2014/05/06 14:40:53 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2013 Thomas Bernard
@@ -9,6 +9,7 @@
 #define GETIFADDR_H_INCLUDED
 
 struct in_addr;
+struct in6_addr;
 
 /* getifaddr()
  * take a network interface name and write the
@@ -18,6 +19,9 @@ int
 getifaddr(const char * ifname, char * buf, int len,
           struct in_addr * addr, struct in_addr * mask);
 
+int
+getifaddr_in6(const char * ifname, int af, struct in6_addr* addr);
+
 /* find a non link local IP v6 address for the interface.
  * if ifname is NULL, look for all interfaces */
 int
