@@ -1,12 +1,12 @@
-/* $Id: macros.h,v 1.1 2012/04/30 20:37:56 nanard Exp $ */
+/* $Id: macros.h,v 1.2 2012/09/27 16:00:10 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2012 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __MACROS_H__
-#define __MACROS_H__
+#ifndef MACROS_H_INCLUDED
+#define MACROS_H_INCLUDED
 
 #define UNUSED(arg)	(void)(arg)
 
