@@ -1,7 +1,7 @@
-/* $Id: miniupnpd.c,v 1.176 2013/06/13 13:21:29 nanard Exp $ */
+/* $Id: miniupnpd.c,v 1.200 2014/05/22 07:56:34 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2013 Thomas Bernard
+ * (c) 2006-2014 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -63,15 +63,21 @@
 #include "miniupnpdtypes.h"
 #include "daemonize.h"
 #include "upnpevents.h"
+#include "asyncsendto.h"
 #ifdef ENABLE_NATPMP
 #include "natpmp.h"
+#ifdef ENABLE_PCP
+#include "pcpserver.h"
+#else
+#define PCP_MAX_LEN 32
+#endif
 #endif
 #include "commonrdr.h"
 #include "upnputils.h"
 #ifdef USE_IFACEWATCHER
 #include "ifacewatcher.h"
 #endif
-#ifdef ENABLE_6FC_SERVICE
+#ifdef ENABLE_UPNPPINHOLE
 #ifdef USE_NETFILTER
 void init_iptpinhole(void);
 #endif
@@ -106,24 +112,40 @@ volatile sig_atomic_t should_send_public_address_change_notif = 0;
 /* OpenAndConfHTTPSocket() :
  * setup the socket used to handle incoming HTTP connections. */
 static int
-OpenAndConfHTTPSocket(unsigned short port)
+#ifdef ENABLE_IPV6
+OpenAndConfHTTPSocket(unsigned short * port, int ipv6)
+#else
+OpenAndConfHTTPSocket(unsigned short * port)
+#endif
 {
 	int s;
 	int i = 1;
 #ifdef ENABLE_IPV6
-	struct sockaddr_in6 listenname;
+	struct sockaddr_in6 listenname6;
+	struct sockaddr_in listenname4;
 #else
 	struct sockaddr_in listenname;
 #endif
 	socklen_t listenname_len;
 
-	if( (s = socket(
+	s = socket(
 #ifdef ENABLE_IPV6
-	                PF_INET6,
+	           ipv6 ? PF_INET6 : PF_INET,
 #else
-	                PF_INET,
+	           PF_INET,
 #endif
-	                SOCK_STREAM, 0)) < 0)
+	           SOCK_STREAM, 0);
+#ifdef ENABLE_IPV6
+	if(s < 0 && ipv6 && errno == EAFNOSUPPORT)
+	{
+		/* the system doesn't support IPV6 */
+		syslog(LOG_WARNING, "socket(PF_INET6, ...) failed with EAFNOSUPPORT, disabling IPv6");
+		SETFLAG(IPV6DISABLEDMASK);
+		ipv6 = 0;
+		s = socket(PF_INET, SOCK_STREAM, 0);
+	}
+#endif
+	if(s < 0)
 	{
 		syslog(LOG_ERR, "socket(http): %m");
 		return -1;
@@ -148,19 +170,35 @@ OpenAndConfHTTPSocket(unsigned short port)
 	}
 
 #ifdef ENABLE_IPV6
-	memset(&listenname, 0, sizeof(struct sockaddr_in6));
-	listenname.sin6_family = AF_INET6;
-	listenname.sin6_port = htons(port);
-	listenname.sin6_addr = in6addr_any;
-	listenname_len =  sizeof(struct sockaddr_in6);
+	if(ipv6)
+	{
+		memset(&listenname6, 0, sizeof(struct sockaddr_in6));
+		listenname6.sin6_family = AF_INET6;
+		listenname6.sin6_port = htons(*port);
+		listenname6.sin6_addr = ipv6_bind_addr;
+		listenname_len =  sizeof(struct sockaddr_in6);
+	} else {
+		memset(&listenname4, 0, sizeof(struct sockaddr_in));
+		listenname4.sin_family = AF_INET;
+		listenname4.sin_port = htons(*port);
+		listenname4.sin_addr.s_addr = htonl(INADDR_ANY);
+		listenname_len =  sizeof(struct sockaddr_in);
+	}
 #else
+	memset(&listenname, 0, sizeof(struct sockaddr_in));
 	listenname.sin_family = AF_INET;
-	listenname.sin_port = htons(port);
+	listenname.sin_port = htons(*port);
 	listenname.sin_addr.s_addr = htonl(INADDR_ANY);
 	listenname_len =  sizeof(struct sockaddr_in);
 #endif
 
+#ifdef ENABLE_IPV6
+	if(bind(s,
+	        ipv6 ? (struct sockaddr *)&listenname6 : (struct sockaddr *)&listenname4,
+	        listenname_len) < 0)
+#else
 	if(bind(s, (struct sockaddr *)&listenname, listenname_len) < 0)
+#endif
 	{
 		syslog(LOG_ERR, "bind(http): %m");
 		close(s);
@@ -174,8 +212,110 @@ OpenAndConfHTTPSocket(unsigned short port)
 		return -1;
 	}
 
+	if(*port == 0) {
+#ifdef ENABLE_IPV6
+		if(ipv6) {
+			struct sockaddr_in6 sockinfo;
+			socklen_t len = sizeof(struct sockaddr_in6);
+			if (getsockname(s, (struct sockaddr *)&sockinfo, &len) < 0) {
+				syslog(LOG_ERR, "getsockname(): %m");
+			} else {
+				*port = ntohs(sockinfo.sin6_port);
+			}
+		} else {
+#endif /* ENABLE_IPV6 */
+			struct sockaddr_in sockinfo;
+			socklen_t len = sizeof(struct sockaddr_in);
+			if (getsockname(s, (struct sockaddr *)&sockinfo, &len) < 0) {
+				syslog(LOG_ERR, "getsockname(): %m");
+			} else {
+				*port = ntohs(sockinfo.sin_port);
+			}
+#ifdef ENABLE_IPV6
+		}
+#endif /* ENABLE_IPV6 */
+	}
 	return s;
 }
+
+static struct upnphttp *
+ProcessIncomingHTTP(int shttpl, const char * protocol)
+{
+	int shttp;
+	socklen_t clientnamelen;
+#ifdef ENABLE_IPV6
+	struct sockaddr_storage clientname;
+	clientnamelen = sizeof(struct sockaddr_storage);
+#else
+	struct sockaddr_in clientname;
+	clientnamelen = sizeof(struct sockaddr_in);
+#endif
+	shttp = accept(shttpl, (struct sockaddr *)&clientname, &clientnamelen);
+	if(shttp<0)
+	{
+		/* ignore EAGAIN, EWOULDBLOCK, EINTR, we just try again later */
+		if(errno != EAGAIN && errno != EWOULDBLOCK && errno != EINTR)
+			syslog(LOG_ERR, "accept(http): %m");
+	}
+	else
+	{
+		struct upnphttp * tmp = 0;
+		char addr_str[64];
+
+		sockaddr_to_string((struct sockaddr *)&clientname, addr_str, sizeof(addr_str));
+		syslog(LOG_INFO, "%s connection from %s", protocol, addr_str);
+		if(get_lan_for_peer((struct sockaddr *)&clientname) == NULL)
+		{
+			/* The peer is not a LAN ! */
+			syslog(LOG_WARNING,
+			       "%s peer %s is not from a LAN, closing the connection",
+			       protocol, addr_str);
+			close(shttp);
+		}
+		else
+		{
+			/* Create a new upnphttp object and add it to
+			 * the active upnphttp object list */
+			tmp = New_upnphttp(shttp);
+			if(tmp)
+			{
+#ifdef ENABLE_IPV6
+				if(clientname.ss_family == AF_INET)
+				{
+					tmp->clientaddr = ((struct sockaddr_in *)&clientname)->sin_addr;
+				}
+				else if(clientname.ss_family == AF_INET6)
+				{
+					struct sockaddr_in6 * addr = (struct sockaddr_in6 *)&clientname;
+					if(IN6_IS_ADDR_V4MAPPED(&addr->sin6_addr))
+					{
+						memcpy(&tmp->clientaddr,
+						       &addr->sin6_addr.s6_addr[12],
+						       4);
+					}
+					else
+					{
+						tmp->ipv6 = 1;
+						memcpy(&tmp->clientaddr_v6,
+						       &addr->sin6_addr,
+						       sizeof(struct in6_addr));
+					}
+				}
+#else
+				tmp->clientaddr = clientname.sin_addr;
+#endif
+				return tmp;
+			}
+			else
+			{
+				syslog(LOG_ERR, "New_upnphttp() failed");
+				close(shttp);
+			}
+		}
+	}
+	return NULL;
+}
+
 #ifdef ENABLE_NFQUEUE
 
 int identify_ip_protocol(char *payload) {
@@ -526,6 +666,9 @@ struct runtime_vars {
 	/* LAN IP addresses for SSDP traffic and HTTP */
 	/* moved to global vars */
 	int port;	/* HTTP Port */
+#ifdef ENABLE_HTTPS
+	int https_port;	/* HTTPS Port */
+#endif
 	int notify_interval;	/* seconds between SSDP announces */
 	/* unused rules cleaning related variables : */
 	int clean_ruleset_threshold;	/* threshold for removing unused rules */
@@ -564,6 +707,7 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 		if(getifaddr(lan_addr->ifname, lan_addr->str, sizeof(lan_addr->str),
 		             &lan_addr->addr, &lan_addr->mask) < 0)
 			goto parselan_error;
+		/*printf("%s => %s\n", lan_addr->ifname, lan_addr->str);*/
 	}
 	else
 	{
@@ -581,6 +725,7 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 			p++;
 		if(*p=='.')
 		{
+			/* parse mask in /255.255.255.0 format */
 			while(*p && (*p=='.' || isdigit(*p)))
 				p++;
 			n = p - q;
@@ -593,6 +738,7 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 		}
 		else
 		{
+			/* it is a /24 format */
 			int nbits = atoi(q);
 			if(nbits > 32 || nbits < 0)
 				goto parselan_error;
@@ -633,7 +779,10 @@ parselanaddr(struct lan_addr_s * lan_addr, const char * str)
 	}
 	else
 	{
-		fprintf(stderr, "Warning: please specify LAN network interface by name instead of IPv4 address\n");
+		fprintf(stderr,
+		        "Error: please specify LAN network interface by name instead of IPv4 address : %s\n",
+		        str);
+		return -1;
 	}
 #endif
 	return 0;
@@ -680,7 +829,10 @@ void complete_uuidvalues(void)
  * 5) check and write pid file
  * 6) set startup time stamp
  * 7) compute presentation URL
- * 8) set signal handlers */
+ * 8) set signal handlers
+ * 9) init random generator (srandom())
+ * 10) init redirection engine
+ * 11) reload mapping from leasefile */
 static int
 init(int argc, char * * argv, struct runtime_vars * v)
 {
@@ -719,9 +871,15 @@ init(int argc, char * * argv, struct runtime_vars * v)
 
 	/* set initial values */
 	SETFLAG(ENABLEUPNPMASK);	/* UPnP is enabled by default */
+#ifdef ENABLE_IPV6
+	ipv6_bind_addr = in6addr_any;
+#endif /* ENABLE_IPV6 */
 
 	LIST_INIT(&lan_addrs);
 	v->port = -1;
+#ifdef ENABLE_HTTPS
+	v->https_port = -1;
+#endif
 	v->notify_interval = 30;	/* seconds between SSDP announces */
 	v->clean_ruleset_threshold = 20;
 	v->clean_ruleset_interval = 0;	/* interval between ruleset check. 0=disabled */
@@ -755,15 +913,32 @@ init(int argc, char * * argv, struct runtime_vars * v)
 				}
 				if(parselanaddr(lan_addr, ary_options[i].value) != 0)
 				{
-					fprintf(stderr, "can't parse \"%s\" as valid lan address\n", ary_options[i].value);
+					fprintf(stderr, "can't parse \"%s\" as a valid "
+#ifndef ENABLE_IPV6
+					        "LAN address or "
+#endif
+					        "interface name\n", ary_options[i].value);
 					free(lan_addr);
 					break;
 				}
 				LIST_INSERT_HEAD(&lan_addrs, lan_addr, list);
 				break;
+#ifdef ENABLE_IPV6
+			case UPNPIPV6_LISTENING_IP:
+				if (inet_pton(AF_INET6, ary_options[i].value, &ipv6_bind_addr) < 1)
+				{
+					fprintf(stderr, "can't parse \"%s\" as valid IPv6 listening address", ary_options[i].value);
+				}
+				break;
+#endif /* ENABLE_IPV6 */
 			case UPNPPORT:
 				v->port = atoi(ary_options[i].value);
 				break;
+#ifdef ENABLE_HTTPS
+			case UPNPHTTPSPORT:
+				v->https_port = atoi(ary_options[i].value);
+				break;
+#endif
 			case UPNPBITRATE_UP:
 				upstream_bitrate = strtoul(ary_options[i].value, 0, 0);
 				break;
@@ -773,10 +948,32 @@ init(int argc, char * * argv, struct runtime_vars * v)
 			case UPNPPRESENTATIONURL:
 				presurl = ary_options[i].value;
 				break;
+#ifdef ENABLE_MANUFACTURER_INFO_CONFIGURATION
 			case UPNPFRIENDLY_NAME:
 				strncpy(friendly_name, ary_options[i].value, FRIENDLY_NAME_MAX_LEN);
 				friendly_name[FRIENDLY_NAME_MAX_LEN-1] = '\0';
 				break;
+			case UPNPMANUFACTURER_NAME:
+				strncpy(manufacturer_name, ary_options[i].value, MANUFACTURER_NAME_MAX_LEN);
+				manufacturer_name[MANUFACTURER_NAME_MAX_LEN-1] = '\0';
+				break;
+			case UPNPMANUFACTURER_URL:
+				strncpy(manufacturer_url, ary_options[i].value, MANUFACTURER_URL_MAX_LEN);
+				manufacturer_url[MANUFACTURER_URL_MAX_LEN-1] = '\0';
+				break;
+			case UPNPMODEL_NAME:
+				strncpy(model_name, ary_options[i].value, MODEL_NAME_MAX_LEN);
+				model_name[MODEL_NAME_MAX_LEN-1] = '\0';
+				break;
+			case UPNPMODEL_DESCRIPTION:
+				strncpy(model_description, ary_options[i].value, MODEL_DESCRIPTION_MAX_LEN);
+				model_description[MODEL_DESCRIPTION_MAX_LEN-1] = '\0';
+				break;
+			case UPNPMODEL_URL:
+				strncpy(model_url, ary_options[i].value, MODEL_URL_MAX_LEN);
+				model_url[MODEL_URL_MAX_LEN-1] = '\0';
+				break;
+#endif
 #ifdef USE_NETFILTER
 			case UPNPFORWARDCHAIN:
 				miniupnpd_forward_chain = ary_options[i].value;
@@ -838,6 +1035,24 @@ init(int argc, char * * argv, struct runtime_vars * v)
 					/*enablenatpmp = atoi(ary_options[i].value);*/
 				break;
 #endif
+#ifdef ENABLE_PCP
+			case UPNPPCPMINLIFETIME:
+					min_lifetime = atoi(ary_options[i].value);
+					if (min_lifetime > 120 ) {
+						min_lifetime = 120;
+					}
+				break;
+			case UPNPPCPMAXLIFETIME:
+					max_lifetime = atoi(ary_options[i].value);
+					if (max_lifetime > 86400 ) {
+						max_lifetime = 86400;
+					}
+				break;
+			case UPNPPCPALLOWTHIRDPARTY:
+				if(strcmp(ary_options[i].value, "yes") == 0)
+					SETFLAG(PCP_ALLOWTHIRDPARTYMASK);
+				break;
+#endif
 #ifdef PF_ENABLE_FILTER_RULES
 			case UPNPQUICKRULES:
 				if(strcmp(ary_options[i].value, "no") == 0)
@@ -865,6 +1080,14 @@ init(int argc, char * * argv, struct runtime_vars * v)
 				        optionsfile);
 			}
 		}
+#ifdef ENABLE_PCP
+		/* if lifetimes are inverse */
+		if (min_lifetime >= max_lifetime) {
+			fprintf(stderr, "Minimum lifetime (%lu) is greater than or equal to maximum lifetime (%lu).\n", min_lifetime, max_lifetime);
+			fprintf(stderr, "Check your configuration file.\n");
+			return 1;
+		}
+#endif
 	}
 #endif /* DISABLE_CONFIG_FILE */
 
@@ -902,6 +1125,7 @@ init(int argc, char * * argv, struct runtime_vars * v)
 			} else
 				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
 			break;
+#ifdef ENABLE_MANUFACTURER_INFO_CONFIGURATION
 		case 'z':
 			if(i+1 < argc)
 				strncpy(friendly_name, argv[++i], FRIENDLY_NAME_MAX_LEN);
@@ -909,6 +1133,7 @@ init(int argc, char * * argv, struct runtime_vars * v)
 				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
 			friendly_name[FRIENDLY_NAME_MAX_LEN-1] = '\0';
 			break;
+#endif
 		case 's':
 			if(i+1 < argc)
 				strncpy(serialnumber, argv[++i], SERIALNUMBER_MAX_LEN);
@@ -969,6 +1194,16 @@ init(int argc, char * * argv, struct runtime_vars * v)
 			if(i+1 < argc)
 				v->port = atoi(argv[++i]);
 			else
+				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
+			break;
+#ifdef ENABLE_HTTPS
+		case 'H':
+			if(i+1 < argc)
+				v->https_port = atoi(argv[++i]);
+			else
+				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
+			break;
+#endif
 #ifdef ENABLE_NFQUEUE
 		case 'Q':
 			if(i+1<argc)
@@ -991,8 +1226,6 @@ init(int argc, char * * argv, struct runtime_vars * v)
 			}
 			break;
 #endif
-				fprintf(stderr, "Option -%c takes one argument.\n", argv[i][1]);
-			break;
 		case 'P':
 			if(i+1 < argc)
 				pidfilename = argv[++i];
@@ -1030,7 +1263,11 @@ init(int argc, char * * argv, struct runtime_vars * v)
 				}
 				if(parselanaddr(lan_addr, argv[i]) != 0)
 				{
-					fprintf(stderr, "can't parse \"%s\" as valid lan address\n", argv[i]);
+					fprintf(stderr, "can't parse \"%s\" as a valid "
+#ifndef ENABLE_IPV6
+					        "LAN address or "
+#endif
+					        "interface name\n", argv[i]);
 					free(lan_addr);
 					break;
 				}
@@ -1065,7 +1302,7 @@ init(int argc, char * * argv, struct runtime_vars * v)
 				}
 				if(parselanaddr(lan_addr, val) != 0)
 				{
-					fprintf(stderr, "can't parse \"%s\" as valid lan address\n", val);
+					fprintf(stderr, "can't parse \"%s\" as a valid LAN address or interface name\n", val);
 					free(lan_addr);
 					free(val);
 					break;
@@ -1192,12 +1429,16 @@ init(int argc, char * * argv, struct runtime_vars * v)
 		syslog(LOG_NOTICE, "Failed to set %s handler", "SIGUSR1");
 	}
 
+	/* initialize random number generator */
+	srandom((unsigned int)time(NULL));
+
+	/* initialize redirection engine (and pinholes) */
 	if(init_redirect() < 0)
 	{
 		syslog(LOG_ERR, "Failed to init redirection engine. EXITING");
 		return 1;
 	}
-#ifdef ENABLE_6FC_SERVICE
+#ifdef ENABLE_UPNPPINHOLE
 #ifdef USE_NETFILTER
 	init_iptpinhole();
 #endif
@@ -1224,6 +1465,9 @@ print_usage:
 			"\t\t[-a listening_ip]"
 #else
 			"\t\t[-a listening_ip ext_ip]"
+#endif
+#ifdef ENABLE_HTTPS
+			" [-H https_port]"
 #endif
 			" [-p port] [-d]"
 #if defined(USE_PF) || defined(USE_IPF)
@@ -1236,7 +1480,10 @@ print_usage:
 			"\n"
 			/*"[-l logfile] " not functionnal */
 			"\t\t[-u uuid] [-s serial] [-m model_number] \n"
-			"\t\t[-t notify_interval] [-P pid_filename] [-z fiendly_name]\n"
+			"\t\t[-t notify_interval] [-P pid_filename] "
+#ifdef ENABLE_MANUFACTURER_INFO_CONFIGURATION
+			"[-z fiendly_name]\n"
+#endif
 			"\t\t[-B down up] [-w url] [-r clean_ruleset_interval]\n"
 #ifdef USE_PF
                         "\t\t[-q queue] [-T tag]\n"
@@ -1286,12 +1533,24 @@ main(int argc, char * * argv)
 {
 	int i;
 	int shttpl = -1;	/* socket for HTTP */
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+	int shttpl_v4 = -1;	/* socket for HTTP (ipv4 only) */
+#endif
+#ifdef ENABLE_HTTPS
+	int shttpsl = -1;	/* socket for HTTPS */
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+	int shttpsl_v4 = -1;	/* socket for HTTPS (ipv4 only) */
+#endif
+#endif /* ENABLE_HTTPS */
 	int sudp = -1;		/* IP v4 socket for receiving SSDP */
 #ifdef ENABLE_IPV6
 	int sudpv6 = -1;	/* IP v6 socket for receiving SSDP */
 #endif
 #ifdef ENABLE_NATPMP
-	int * snatpmp = NULL;
+	int * snatpmp = NULL;	/* also used for PCP */
+#endif
+#if defined(ENABLE_IPV6) && defined(ENABLE_PCP)
+	int spcp_v6 = -1;
 #endif
 #ifdef ENABLE_NFQUEUE
 	int nfqh = -1;
@@ -1320,12 +1579,16 @@ main(int argc, char * * argv)
 	struct rule_state * rule_list = 0;
 	struct timeval checktime = {0, 0};
 	struct lan_addr_s * lan_addr;
-#ifdef ENABLE_6FC_SERVICE
+#ifdef ENABLE_UPNPPINHOLE
 	unsigned int next_pinhole_ts;
 #endif
 
 	if(init(argc, argv, &v) != 0)
 		return 1;
+#ifdef ENABLE_HTTPS
+	if(init_ssl() < 0)
+		return 1;
+#endif /* ENABLE_HTTPS */
 	/* count lan addrs */
 	addr_count = 0;
 	for(lan_addr = lan_addrs.lh_first; lan_addr != NULL; lan_addr = lan_addr->list.le_next)
@@ -1362,7 +1625,11 @@ main(int argc, char * * argv)
 
 	syslog(LOG_INFO, "Starting%s%swith external interface %s",
 #ifdef ENABLE_NATPMP
+#ifdef ENABLE_PCP
+	       GETFLAG(ENABLENATPMPMASK) ? " NAT-PMP/PCP " : " ",
+#else
 	       GETFLAG(ENABLENATPMPMASK) ? " NAT-PMP " : " ",
+#endif
 #else
 	       " ",
 #endif
@@ -1371,31 +1638,64 @@ main(int argc, char * * argv)
 
 	if(GETFLAG(ENABLEUPNPMASK))
 	{
-
+		unsigned short listen_port;
+		listen_port = (v.port > 0) ? v.port : 0;
 		/* open socket for HTTP connections. Listen on the 1st LAN address */
-		shttpl = OpenAndConfHTTPSocket((v.port > 0) ? v.port : 0);
+#ifdef ENABLE_IPV6
+		shttpl = OpenAndConfHTTPSocket(&listen_port, 1);
+#else /* ENABLE_IPV6 */
+		shttpl = OpenAndConfHTTPSocket(&listen_port);
+#endif /* ENABLE_IPV6 */
 		if(shttpl < 0)
 		{
 			syslog(LOG_ERR, "Failed to open socket for HTTP. EXITING");
 			return 1;
 		}
-		if(v.port <= 0) {
-			struct sockaddr_in sockinfo;
-			socklen_t len = sizeof(struct sockaddr_in);
-			if (getsockname(shttpl, (struct sockaddr *)&sockinfo, &len) < 0) {
-				syslog(LOG_ERR, "getsockname(): %m");
+		v.port = listen_port;
+		syslog(LOG_NOTICE, "HTTP listening on port %d", v.port);
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+		if(!GETFLAG(IPV6DISABLEDMASK))
+		{
+			shttpl_v4 =  OpenAndConfHTTPSocket(&listen_port, 0);
+			if(shttpl_v4 < 0)
+			{
+				syslog(LOG_ERR, "Failed to open socket for HTTP on port %hu (IPv4). EXITING", v.port);
 				return 1;
 			}
-			v.port = ntohs(sockinfo.sin_port);
 		}
-		syslog(LOG_NOTICE, "HTTP listening on port %d", v.port);
+#endif /* V6SOCKETS_ARE_V6ONLY */
+#ifdef ENABLE_HTTPS
+		/* https */
+		listen_port = (v.https_port > 0) ? v.https_port : 0;
+#ifdef ENABLE_IPV6
+		shttpsl = OpenAndConfHTTPSocket(&listen_port, 1);
+#else /* ENABLE_IPV6 */
+		shttpsl = OpenAndConfHTTPSocket(&listen_port);
+#endif /* ENABLE_IPV6 */
+		if(shttpl < 0)
+		{
+			syslog(LOG_ERR, "Failed to open socket for HTTPS. EXITING");
+			return 1;
+		}
+		v.https_port = listen_port;
+		syslog(LOG_NOTICE, "HTTPS listening on port %d", v.https_port);
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+		shttpsl_v4 =  OpenAndConfHTTPSocket(&listen_port, 0);
+		if(shttpsl_v4 < 0)
+		{
+			syslog(LOG_ERR, "Failed to open socket for HTTPS on port %hu (IPv4). EXITING", v.https_port);
+			return 1;
+		}
+#endif /* V6SOCKETS_ARE_V6ONLY */
+#endif /* ENABLE_HTTPS */
 #ifdef ENABLE_IPV6
 		if(find_ipv6_addr(NULL, ipv6_addr_for_http_with_brackets, sizeof(ipv6_addr_for_http_with_brackets)) > 0) {
 			syslog(LOG_NOTICE, "HTTP IPv6 address given to control points : %s",
 			       ipv6_addr_for_http_with_brackets);
 		} else {
 			memcpy(ipv6_addr_for_http_with_brackets, "[::1]", 6);
-			syslog(LOG_WARNING, "no HTTP IPv6 address");
+			syslog(LOG_WARNING, "no HTTP IPv6 address, disabling IPv6");
+			SETFLAG(IPV6DISABLEDMASK);
 		}
 #endif
 
@@ -1410,10 +1710,13 @@ main(int argc, char * * argv)
 			}
 		}
 #ifdef ENABLE_IPV6
-		sudpv6 = OpenAndConfSSDPReceiveSocket(1);
-		if(sudpv6 < 0)
+		if(!GETFLAG(IPV6DISABLEDMASK))
 		{
-			syslog(LOG_WARNING, "Failed to open socket for receiving SSDP (IP v6).");
+			sudpv6 = OpenAndConfSSDPReceiveSocket(1);
+			if(sudpv6 < 0)
+			{
+				syslog(LOG_WARNING, "Failed to open socket for receiving SSDP (IP v6).");
+			}
 		}
 #endif
 
@@ -1443,18 +1746,28 @@ main(int argc, char * * argv)
 	if(GETFLAG(ENABLENATPMPMASK))
 	{
 		if(OpenAndConfNATPMPSockets(snatpmp) < 0)
+#ifdef ENABLE_PCP
+		{
+			syslog(LOG_ERR, "Failed to open sockets for NAT-PMP/PCP.");
+		} else {
+			syslog(LOG_NOTICE, "Listening for NAT-PMP/PCP traffic on port %u",
+			       NATPMP_PORT);
+		}
+#else
 		{
 			syslog(LOG_ERR, "Failed to open sockets for NAT PMP.");
 		} else {
 			syslog(LOG_NOTICE, "Listening for NAT-PMP traffic on port %u",
 			       NATPMP_PORT);
 		}
-#if 0
-		ScanNATPMPforExpiration();
 #endif
 	}
 #endif
 
+#if defined(ENABLE_IPV6) && defined(ENABLE_PCP)
+	spcp_v6 = OpenAndConfPCPv6Socket();
+#endif
+
 	/* for miniupnpdctl */
 #ifdef USE_MINIUPNPDCTL
 	sctl = OpenAndConfCtlUnixSocket("/var/run/miniupnpd.ctl");
@@ -1483,7 +1796,7 @@ main(int argc, char * * argv)
 		/* send public address change notifications if needed */
 		if(should_send_public_address_change_notif)
 		{
-			syslog(LOG_DEBUG, "should send external iface address change notification(s)");
+			syslog(LOG_INFO, "should send external iface address change notification(s)");
 #ifdef ENABLE_NATPMP
 			if(GETFLAG(ENABLENATPMPMASK))
 				SendNATPMPPublicAddressChangeNotification(snatpmp, addr_count);
@@ -1512,6 +1825,9 @@ main(int argc, char * * argv)
 				if (GETFLAG(ENABLEUPNPMASK))
 					SendSSDPNotifies2(snotify,
 				                  (unsigned short)v.port,
+#ifdef ENABLE_HTTPS
+					              (unsigned short)v.https_port,
+#endif
 				                  v.notify_interval << 1);
 				memcpy(&lasttimeofday, &timeofday, sizeof(struct timeval));
 				timeout.tv_sec = v.notify_interval;
@@ -1564,29 +1880,7 @@ main(int argc, char * * argv)
 			syslog(LOG_DEBUG, "setting timeout to %u sec",
 			       (unsigned)timeout.tv_sec);
 		}
-#ifdef ENABLE_NATPMP
-#if 0
-		/* Remove expired NAT-PMP mappings */
-		while(nextnatpmptoclean_timestamp
-		     && (timeofday.tv_sec >= nextnatpmptoclean_timestamp + startup_time))
-		{
-			/*syslog(LOG_DEBUG, "cleaning expired NAT-PMP mappings");*/
-			if(CleanExpiredNATPMP() < 0) {
-				syslog(LOG_ERR, "CleanExpiredNATPMP() failed");
-				break;
-			}
-		}
-		if(nextnatpmptoclean_timestamp
-		  && timeout.tv_sec >= (nextnatpmptoclean_timestamp + startup_time - timeofday.tv_sec))
-		{
-			/*syslog(LOG_DEBUG, "setting timeout to %d sec",
-			       nextnatpmptoclean_timestamp + startup_time - timeofday.tv_sec);*/
-			timeout.tv_sec = nextnatpmptoclean_timestamp + startup_time - timeofday.tv_sec;
-			timeout.tv_usec = 0;
-		}
-#endif
-#endif
-#ifdef ENABLE_6FC_SERVICE
+#ifdef ENABLE_UPNPPINHOLE
 		/* Clean up expired IPv6 PinHoles */
 		next_pinhole_ts = 0;
 		upnp_clean_expired_pinholes(&next_pinhole_ts);
@@ -1595,7 +1889,7 @@ main(int argc, char * * argv)
 			timeout.tv_sec = next_pinhole_ts - timeofday.tv_sec;
 			timeout.tv_usec = 0;
 		}
-#endif
+#endif /* ENABLE_UPNPPINHOLE */
 
 		/* select open sockets (SSDP, HTTP listen, and all HTTP soap sockets) */
 		FD_ZERO(&readset);
@@ -1618,6 +1912,27 @@ main(int argc, char * * argv)
 			FD_SET(shttpl, &readset);
 			max_fd = MAX( max_fd, shttpl);
 		}
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+		if (shttpl_v4 >= 0)
+		{
+			FD_SET(shttpl_v4, &readset);
+			max_fd = MAX( max_fd, shttpl_v4);
+		}
+#endif
+#ifdef ENABLE_HTTPS
+		if (shttpsl >= 0)
+		{
+			FD_SET(shttpsl, &readset);
+			max_fd = MAX( max_fd, shttpsl);
+		}
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+		if (shttpsl_v4 >= 0)
+		{
+			FD_SET(shttpsl_v4, &readset);
+			max_fd = MAX( max_fd, shttpsl_v4);
+		}
+#endif
+#endif /* ENABLE_HTTPS */
 #ifdef ENABLE_IPV6
 		if (sudpv6 >= 0)
 		{
@@ -1664,6 +1979,12 @@ main(int argc, char * * argv)
 			}
 		}
 #endif
+#if defined(ENABLE_IPV6) && defined(ENABLE_PCP)
+		if(spcp_v6 >= 0) {
+			FD_SET(spcp_v6, &readset);
+			max_fd = MAX(max_fd, spcp_v6);
+		}
+#endif
 #ifdef USE_MINIUPNPDCTL
 		if(sctl >= 0) {
 			FD_SET(sctl, &readset);
@@ -1683,6 +2004,38 @@ main(int argc, char * * argv)
 		upnpevents_selectfds(&readset, &writeset, &max_fd);
 #endif
 
+		/* queued "sendto" */
+		{
+			struct timeval next_send;
+			i = get_next_scheduled_send(&next_send);
+			if(i > 0) {
+#ifdef DEBUG
+				syslog(LOG_DEBUG, "%d queued sendto", i);
+#endif
+				i = get_sendto_fds(&writeset, &max_fd, &timeofday);
+				if(timeofday.tv_sec > next_send.tv_sec ||
+				   (timeofday.tv_sec == next_send.tv_sec && timeofday.tv_usec >= next_send.tv_usec)) {
+					if(i > 0) {
+						timeout.tv_sec = 0;
+						timeout.tv_usec = 0;
+					}
+				} else {
+					struct timeval tmp_timeout;
+					tmp_timeout.tv_sec = (next_send.tv_sec - timeofday.tv_sec);
+					tmp_timeout.tv_usec = (next_send.tv_usec - timeofday.tv_usec);
+					if(tmp_timeout.tv_usec < 0) {
+						tmp_timeout.tv_usec += 1000000;
+						tmp_timeout.tv_sec--;
+					}
+					if(timeout.tv_sec > tmp_timeout.tv_sec
+					   || (timeout.tv_sec == tmp_timeout.tv_sec && timeout.tv_usec > tmp_timeout.tv_usec)) {
+						timeout.tv_sec = tmp_timeout.tv_sec;
+						timeout.tv_usec = tmp_timeout.tv_usec;
+					}
+				}
+			}
+		}
+
 		if(select(max_fd+1, &readset, &writeset, 0, &timeout) < 0)
 		{
 			if(quitting) goto shutdown;
@@ -1691,6 +2044,10 @@ main(int argc, char * * argv)
 			syslog(LOG_ERR, "Failed to select open sockets. EXITING");
 			return 1;	/* very serious cause of error */
 		}
+		i = try_sendto(&writeset);
+		if(i < 0) {
+			syslog(LOG_ERR, "try_sendto failed to send %d packets", -i);
+		}
 #ifdef USE_MINIUPNPDCTL
 		for(ectl = ctllisthead.lh_first; ectl;)
 		{
@@ -1763,21 +2120,75 @@ main(int argc, char * * argv)
 		{
 			if((snatpmp[i] >= 0) && FD_ISSET(snatpmp[i], &readset))
 			{
-				ProcessIncomingNATPMPPacket(snatpmp[i]);
+				unsigned char msg_buff[PCP_MAX_LEN];
+				struct sockaddr_in senderaddr;
+				socklen_t senderaddrlen;
+				int len;
+				memset(msg_buff, 0, PCP_MAX_LEN);
+				senderaddrlen = sizeof(senderaddr);
+				len = ReceiveNATPMPOrPCPPacket(snatpmp[i],
+				                               (struct sockaddr *)&senderaddr,
+				                               &senderaddrlen,
+				                               NULL,
+				                               msg_buff, sizeof(msg_buff));
+				if (len < 1)
+					continue;
+#ifdef ENABLE_PCP
+				if (msg_buff[0]==0) {  /* version equals to 0 -> means NAT-PMP */
+					ProcessIncomingNATPMPPacket(snatpmp[i], msg_buff, len,
+					                            &senderaddr);
+				} else { /* everything else can be PCP */
+					ProcessIncomingPCPPacket(snatpmp[i], msg_buff, len,
+					                         (struct sockaddr *)&senderaddr, NULL);
+				}
+
+#else
+				ProcessIncomingNATPMPPacket(snatpmp[i], msg_buff, len, &senderaddr);
+#endif
 			}
 		}
+#endif
+#if defined(ENABLE_IPV6) && defined(ENABLE_PCP)
+		/* in IPv6, only PCP is supported, not NAT-PMP */
+		if(spcp_v6 >= 0 && FD_ISSET(spcp_v6, &readset))
+		{
+			unsigned char msg_buff[PCP_MAX_LEN];
+			struct sockaddr_in6 senderaddr;
+			socklen_t senderaddrlen;
+			struct sockaddr_in6 receiveraddr;
+			int len;
+			memset(msg_buff, 0, PCP_MAX_LEN);
+			senderaddrlen = sizeof(senderaddr);
+			len = ReceiveNATPMPOrPCPPacket(spcp_v6,
+			                               (struct sockaddr *)&senderaddr,
+			                               &senderaddrlen,
+			                               &receiveraddr,
+			                               msg_buff, sizeof(msg_buff));
+			if(len >= 1)
+				ProcessIncomingPCPPacket(spcp_v6, msg_buff, len,
+				                         (struct sockaddr *)&senderaddr,
+				                         &receiveraddr);
+		}
 #endif
 		/* process SSDP packets */
 		if(sudp >= 0 && FD_ISSET(sudp, &readset))
 		{
 			/*syslog(LOG_INFO, "Received UDP Packet");*/
+#ifdef ENABLE_HTTPS
+			ProcessSSDPRequest(sudp, (unsigned short)v.port, (unsigned short)v.https_port);
+#else
 			ProcessSSDPRequest(sudp, (unsigned short)v.port);
+#endif
 		}
 #ifdef ENABLE_IPV6
 		if(sudpv6 >= 0 && FD_ISSET(sudpv6, &readset))
 		{
 			syslog(LOG_INFO, "Received UDP Packet (IPv6)");
+#ifdef ENABLE_HTTPS
+			ProcessSSDPRequest(sudpv6, (unsigned short)v.port, (unsigned short)v.https_port);
+#else
 			ProcessSSDPRequest(sudpv6, (unsigned short)v.port);
+#endif
 		}
 #endif
 #ifdef USE_IFACEWATCHER
@@ -1802,79 +2213,48 @@ main(int argc, char * * argv)
 		/* process incoming HTTP connections */
 		if(shttpl >= 0 && FD_ISSET(shttpl, &readset))
 		{
-			int shttp;
-			socklen_t clientnamelen;
-#ifdef ENABLE_IPV6
-			struct sockaddr_storage clientname;
-			clientnamelen = sizeof(struct sockaddr_storage);
-#else
-			struct sockaddr_in clientname;
-			clientnamelen = sizeof(struct sockaddr_in);
-#endif
-			shttp = accept(shttpl, (struct sockaddr *)&clientname, &clientnamelen);
-			if(shttp<0)
+			struct upnphttp * tmp;
+			tmp = ProcessIncomingHTTP(shttpl, "HTTP");
+			if(tmp)
 			{
-				/* ignore EAGAIN, EWOULDBLOCK, EINTR, we just try again later */
-				if(errno != EAGAIN && errno != EWOULDBLOCK && errno != EINTR)
-					syslog(LOG_ERR, "accept(http): %m");
-			}
-			else
-			{
-				struct upnphttp * tmp = 0;
-				char addr_str[64];
-
-				sockaddr_to_string((struct sockaddr *)&clientname, addr_str, sizeof(addr_str));
-				syslog(LOG_INFO, "HTTP connection from %s", addr_str);
-				if(get_lan_for_peer((struct sockaddr *)&clientname) == NULL)
-				{
-					/* The peer is not a LAN ! */
-					syslog(LOG_WARNING,
-					       "HTTP peer %s is not from a LAN, closing the connection",
-					       addr_str);
-					close(shttp);
-				}
-				else
-				{
-					/* Create a new upnphttp object and add it to
-					 * the active upnphttp object list */
-					tmp = New_upnphttp(shttp);
-					if(tmp)
-					{
-#ifdef ENABLE_IPV6
-						if(clientname.ss_family == AF_INET)
-						{
-							tmp->clientaddr = ((struct sockaddr_in *)&clientname)->sin_addr;
-						}
-						else if(clientname.ss_family == AF_INET6)
-						{
-							struct sockaddr_in6 * addr = (struct sockaddr_in6 *)&clientname;
-							if(IN6_IS_ADDR_V4MAPPED(&addr->sin6_addr))
-							{
-								memcpy(&tmp->clientaddr,
-								       &addr->sin6_addr.s6_addr[12],
-								       4);
-							}
-							else
-							{
-								tmp->ipv6 = 1;
-								memcpy(&tmp->clientaddr_v6,
-								       &addr->sin6_addr,
-								       sizeof(struct in6_addr));
-							}
-						}
-#else
-						tmp->clientaddr = clientname.sin_addr;
-#endif
-						LIST_INSERT_HEAD(&upnphttphead, tmp, entries);
-					}
-					else
-					{
-						syslog(LOG_ERR, "New_upnphttp() failed");
-						close(shttp);
-					}
-				}
+				LIST_INSERT_HEAD(&upnphttphead, tmp, entries);
 			}
 		}
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+		if(shttpl_v4 >= 0 && FD_ISSET(shttpl_v4, &readset))
+		{
+			struct upnphttp * tmp;
+			tmp = ProcessIncomingHTTP(shttpl_v4, "HTTP");
+			if(tmp)
+			{
+				LIST_INSERT_HEAD(&upnphttphead, tmp, entries);
+			}
+		}
+#endif
+#ifdef ENABLE_HTTPS
+		if(shttpsl >= 0 && FD_ISSET(shttpsl, &readset))
+		{
+			struct upnphttp * tmp;
+			tmp = ProcessIncomingHTTP(shttpsl, "HTTPS");
+			if(tmp)
+			{
+				InitSSL_upnphttp(tmp);
+				LIST_INSERT_HEAD(&upnphttphead, tmp, entries);
+			}
+		}
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+		if(shttpsl_v4 >= 0 && FD_ISSET(shttpsl_v4, &readset))
+		{
+			struct upnphttp * tmp;
+			tmp = ProcessIncomingHTTP(shttpsl_v4, "HTTPS");
+			if(tmp)
+			{
+				InitSSL_upnphttp(tmp);
+				LIST_INSERT_HEAD(&upnphttphead, tmp, entries);
+			}
+		}
+#endif
+#endif /* ENABLE_HTTPS */
 #ifdef ENABLE_NFQUEUE
 		/* process NFQ packets */
 		if(nfqh >= 0 && FD_ISSET(nfqh, &readset))
@@ -1898,6 +2278,22 @@ main(int argc, char * * argv)
 	}	/* end of main loop */
 
 shutdown:
+	syslog(LOG_NOTICE, "shutting down MiniUPnPd");
+	/* send good-bye */
+	if (GETFLAG(ENABLEUPNPMASK))
+	{
+#ifndef ENABLE_IPV6
+		if(SendSSDPGoodbye(snotify, addr_count) < 0)
+#else
+		if(SendSSDPGoodbye(snotify, addr_count * 2) < 0)
+#endif
+		{
+			syslog(LOG_ERR, "Failed to broadcast good-bye notifications");
+		}
+	}
+	/* try to send pending packets */
+	finalize_sendto();
+
 	/* close out open sockets */
 	while(upnphttphead.lh_first != NULL)
 	{
@@ -1908,6 +2304,9 @@ shutdown:
 
 	if (sudp >= 0) close(sudp);
 	if (shttpl >= 0) close(shttpl);
+#if defined(V6SOCKETS_ARE_V6ONLY) && defined(ENABLE_IPV6)
+	if (shttpl_v4 >= 0) close(shttpl_v4);
+#endif
 #ifdef ENABLE_IPV6
 	if (sudpv6 >= 0) close(sudpv6);
 #endif
@@ -1923,6 +2322,13 @@ shutdown:
 		}
 	}
 #endif
+#if defined(ENABLE_IPV6) && defined(ENABLE_PCP)
+	if(spcp_v6 >= 0)
+	{
+		close(spcp_v6);
+		spcp_v6 = -1;
+	}
+#endif
 #ifdef USE_MINIUPNPDCTL
 	if(sctl>=0)
 	{
@@ -1937,14 +2343,6 @@ shutdown:
 
 	if (GETFLAG(ENABLEUPNPMASK))
 	{
-#ifndef ENABLE_IPV6
-		if(SendSSDPGoodbye(snotify, addr_count) < 0)
-#else
-		if(SendSSDPGoodbye(snotify, addr_count * 2) < 0)
-#endif
-		{
-			syslog(LOG_ERR, "Failed to broadcast good-bye notifications");
-		}
 #ifndef ENABLE_IPV6
 		for(i = 0; i < addr_count; i++)
 #else
@@ -1953,6 +2351,7 @@ shutdown:
 			close(snotify[i]);
 	}
 
+	/* remove pidfile */
 	if(pidfilename && (unlink(pidfilename) < 0))
 	{
 		syslog(LOG_ERR, "Failed to remove pidfile %s: %m", pidfilename);
@@ -1966,6 +2365,9 @@ shutdown:
 		free(lan_addr);
 	}
 
+#ifdef ENABLE_HTTPS
+	free_ssl();
+#endif
 #ifdef ENABLE_NATPMP
 	free(snatpmp);
 #endif
