@@ -1,11 +1,11 @@
-/* $Id: miniupnpdtypes.h,v 1.4 2012/04/06 15:27:21 nanard Exp $ */
+/* $Id: miniupnpdtypes.h,v 1.5 2012/09/27 15:47:15 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2012 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
-#ifndef __MINIUPNPDTYPES_H__
-#define __MINIUPNPDTYPES_H__
+#ifndef MINIUPNPDTYPES_H_INCLUDED
+#define MINIUPNPDTYPES_H_INCLUDED
 
 #include "config.h"
 #include <netinet/in.h>
