@@ -1,8 +1,8 @@
-/* $Id: options.h,v 1.22 2012/09/27 15:47:15 nanard Exp $ */
+/* $Id: options.h,v 1.26 2014/05/22 07:52:45 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * author: Ryan Wagoner
- * (c) 2006-2012 Thomas Bernard
+ * (c) 2006-2014 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -18,11 +18,24 @@ enum upnpconfigoptions {
 	UPNPEXT_IFNAME = 1,		/* ext_ifname */
 	UPNPEXT_IP,				/* ext_ip */
 	UPNPLISTENING_IP,		/* listening_ip */
-	UPNPPORT,				/* "port" */
+#ifdef ENABLE_IPV6
+	UPNPIPV6_LISTENING_IP,		/* listening address for IPv6 */
+#endif /* ENABLE_IPV6 */
+	UPNPPORT,				/* "port" / "http_port" */
+#ifdef ENABLE_HTTPS
+	UPNPHTTPSPORT,			/* "https_port" */
+#endif
 	UPNPBITRATE_UP,			/* "bitrate_up" */
 	UPNPBITRATE_DOWN,		/* "bitrate_down" */
 	UPNPPRESENTATIONURL,	/* presentation_url */
+#ifdef ENABLE_MANUFACTURER_INFO_CONFIGURATION
 	UPNPFRIENDLY_NAME,		/* "friendly_name" */
+	UPNPMANUFACTURER_NAME,	/* "manufacturer_name" */
+	UPNPMANUFACTURER_URL,	/* "manufacturer_url" */
+	UPNPMODEL_NAME,	/* "model_name" */
+	UPNPMODEL_DESCRIPTION,	/* "model_description" */
+	UPNPMODEL_URL,	/* "model_url" */
+#endif
 	UPNPNOTIFY_INTERVAL,	/* notify_interval */
 	UPNPSYSTEM_UPTIME,		/* "system_uptime" */
 	UPNPPACKET_LOG,			/* "packet_log" */
@@ -32,6 +45,9 @@ enum upnpconfigoptions {
 	UPNPCLEANTHRESHOLD,		/* clean_ruleset_threshold */
 	UPNPCLEANINTERVAL,		/* clean_ruleset_interval */
 	UPNPENABLENATPMP,		/* enable_natpmp */
+	UPNPPCPMINLIFETIME,		/* minimum lifetime for PCP mapping */
+	UPNPPCPMAXLIFETIME,		/* maximum lifetime for PCP mapping */
+	UPNPPCPALLOWTHIRDPARTY,		/* allow third-party requests */
 #ifdef USE_NETFILTER
 	UPNPFORWARDCHAIN,
 	UPNPNATCHAIN,
