@@ -1,4 +1,4 @@
-/* $Id: pfpinhole.h,v 1.10 2012/09/27 15:44:10 nanard Exp $ */
+/* $Id: pfpinhole.h,v 1.12 2014/05/15 09:27:21 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2012 Thomas Bernard
@@ -8,11 +8,12 @@
 #ifndef PFPINHOLE_H_INCLUDED
 #define PFPINHOLE_H_INCLUDED
 
-#ifdef ENABLE_6FC_SERVICE
+#ifdef ENABLE_UPNPPINHOLE
+
 int add_pinhole(const char * ifname,
                 const char * rem_host, unsigned short rem_port,
                 const char * int_client, unsigned short int_port,
-                int proto, unsigned int timestamp);
+                int proto, const char * desc, unsigned int timestamp);
 
 int delete_pinhole(unsigned short uid);
 
@@ -20,14 +21,15 @@ int
 get_pinhole_info(unsigned short uid,
                  char * rem_host, int rem_hostlen, unsigned short * rem_port,
                  char * int_client, int int_clientlen, unsigned short * int_port,
-                 int * proto, unsigned int * timestamp,
+                 int * proto, char * desc, int desclen,
+                 unsigned int * timestamp,
                  u_int64_t * packets, u_int64_t * bytes);
 
 int update_pinhole(unsigned short uid, unsigned int timestamp);
 
 int clean_pinhole_list(unsigned int * next_timestamp);
 
-#endif
+#endif /* ENABLE_UPNPPINHOLE */
 
 #endif
 
