@@ -1,7 +1,7 @@
-/* $Id: testupnpdescgen.c,v 1.30 2013/06/13 13:21:30 nanard Exp $ */
+/* $Id: testupnpdescgen.c,v 1.32 2014/03/10 11:04:52 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2013 Thomas Bernard
+ * (c) 2006-2014 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -16,6 +16,8 @@
 #include "macros.h"
 #include "config.h"
 #include "upnpdescgen.h"
+#include "upnpdescstrings.h"
+#include "getifaddr.h"
 
 char uuidvalue_igd[] = "uuid:12345678-0000-0000-0000-000000abcd01";
 char uuidvalue_wan[] = "uuid:12345678-0000-0000-0000-000000abcd02";
@@ -24,19 +26,25 @@ char serialnumber[] = "12345678";
 char modelnumber[] = "1";
 char presentationurl[] = "http://192.168.0.1:8080/";
 /*char presentationurl[] = "";*/
+#ifdef ENABLE_MANUFACTURER_INFO_CONFIGURATION
 char friendly_name[] = OS_NAME " router";
+char manufacturer_name[] = ROOTDEV_MANUFACTURER;
+char manufacturer_url[] = ROOTDEV_MANUFACTURERURL;
+char model_name[] = ROOTDEV_MODELNAME;
+char model_description[] = ROOTDEV_MODELDESCRIPTION;
+char model_url[] = ROOTDEV_MODELURL;
+#endif
 
 char * use_ext_ip_addr = NULL;
 const char * ext_if_name = "eth0";
 
-#ifdef ENABLE_6FC_SERVICE
-int ipv6fc_firewall_enabled = 1;
-int ipv6fc_inbound_pinhole_allowed = 1;
-#endif
+int runtime_flags = 0;
 
-int getifaddr(const char * ifname, char * buf, int len)
+int getifaddr(const char * ifname, char * buf, int len, struct in_addr * addr, struct in_addr * mask)
 {
 	UNUSED(ifname);
+	UNUSED(addr);
+	UNUSED(mask);
 	strncpy(buf, "1.2.3.4", len);
 	return 0;
 }
