@@ -1,7 +1,7 @@
-/* $Id: upnpdescgen.c,v 1.75 2013/07/30 06:55:19 nanard Exp $ */
+/* $Id: upnpdescgen.c,v 1.77 2014/03/10 11:04:53 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2013 Thomas Bernard
+ * (c) 2006-2014 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -145,14 +145,25 @@ static const struct XMLElt rootDesc[] =
 /* 5 */
 	{"/deviceType", DEVICE_TYPE_IGD},
 		/* urn:schemas-upnp-org:device:InternetGatewayDevice:1 or 2 */
+#ifdef ENABLE_MANUFACTURER_INFO_CONFIGURATION
 	{"/friendlyName", friendly_name/*ROOTDEV_FRIENDLYNAME*/},	/* required */
-	{"/manufacturer", ROOTDEV_MANUFACTURER},		/* required */
+	{"/manufacturer", manufacturer_name/*ROOTDEV_MANUFACTURER*/},		/* required */
+/* 8 */
+	{"/manufacturerURL", manufacturer_url/*ROOTDEV_MANUFACTURERURL*/},	/* optional */
+	{"/modelDescription", model_description/*ROOTDEV_MODELDESCRIPTION*/}, /* recommended */
+	{"/modelName", model_name/*ROOTDEV_MODELNAME*/},	/* required */
+	{"/modelNumber", modelnumber},
+	{"/modelURL", model_url/*ROOTDEV_MODELURL*/},
+#else
+	{"/friendlyName", ROOTDEV_FRIENDLYNAME},	/* required */
+	{"/manufacturer", ROOTDEV_MANUFACTURER},	/* required */
 /* 8 */
 	{"/manufacturerURL", ROOTDEV_MANUFACTURERURL},	/* optional */
 	{"/modelDescription", ROOTDEV_MODELDESCRIPTION}, /* recommended */
 	{"/modelName", ROOTDEV_MODELNAME},	/* required */
 	{"/modelNumber", modelnumber},
 	{"/modelURL", ROOTDEV_MODELURL},
+#endif
 	{"/serialNumber", serialnumber},
 	{"/UDN", uuidvalue_igd},	/* required */
 	/* see if /UPC is needed. */
@@ -1165,13 +1176,13 @@ genEventVars(int * len, const struct serviceDesc * s)
 			case FIREWALLENABLED_MAGICALVALUE:
 				/* see 2.4.2 of UPnP-gw-WANIPv6FirewallControl-v1-Service.pdf */
 				snprintf(tmp, sizeof(tmp), "%d",
-				         ipv6fc_firewall_enabled);
+				         GETFLAG(IPV6FCFWDISABLEDMASK) ? 0 : 1);
 				str = strcat_str(str, len, &tmplen, tmp);
 				break;
 			case INBOUNDPINHOLEALLOWED_MAGICALVALUE:
 				/* see 2.4.3 of UPnP-gw-WANIPv6FirewallControl-v1-Service.pdf */
 				snprintf(tmp, sizeof(tmp), "%d",
-				         ipv6fc_inbound_pinhole_allowed);
+				         GETFLAG(IPV6FCINBOUNDDISALLOWEDMASK) ? 0 : 1);
 				str = strcat_str(str, len, &tmplen, tmp);
 				break;
 #endif
