@@ -1,7 +1,7 @@
-/* $Id: upnpevents.c,v 1.27 2013/06/13 13:21:30 nanard Exp $ */
+/* $Id: upnpevents.c,v 1.30 2014/03/14 22:26:07 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2008-2013 Thomas Bernard
+ * (c) 2008-2014 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -149,6 +149,11 @@ renewSubscription(const char * sid, int sidlen, int timeout)
 	struct subscriber * sub;
 	for(sub = subscriberlist.lh_first; sub != NULL; sub = sub->entries.le_next) {
 		if((sidlen == 41) && (memcmp(sid, sub->uuid, 41) == 0)) {
+#ifdef UPNP_STRICT
+			/* check if the subscription already timeouted */
+			if(sub->timeout && time(NULL) > sub->timeout)
+				continue;
+#endif
 			sub->timeout = (timeout ? time(NULL) + timeout : 0);
 			return 0;
 		}
@@ -192,6 +197,7 @@ static void
 upnp_event_create_notify(struct subscriber * sub)
 {
 	struct upnp_event_notify * obj;
+	/*struct timeval sock_timeout;*/
 
 	obj = calloc(1, sizeof(struct upnp_event_notify));
 	if(!obj) {
@@ -210,6 +216,22 @@ upnp_event_create_notify(struct subscriber * sub)
 		syslog(LOG_ERR, "%s: socket(): %m", "upnp_event_create_notify");
 		goto error;
 	}
+#if 0 /* does not work for non blocking connect() */
+	/* set timeout to 3 seconds */
+	sock_timeout.tv_sec = 3;
+	sock_timeout.tv_usec = 0;
+	if(setsockopt(obj->s, SOL_SOCKET, SO_RCVTIMEO, &sock_timeout, sizeof(struct timeval)) < 0) {
+		syslog(LOG_WARNING, "%s: setsockopt(SO_RCVTIMEO): %m",
+		       "upnp_event_create_notify");
+	}
+	sock_timeout.tv_sec = 3;
+	sock_timeout.tv_usec = 0;
+	if(setsockopt(obj->s, SOL_SOCKET, SO_SNDTIMEO, &sock_timeout, sizeof(struct timeval)) < 0) {
+		syslog(LOG_WARNING, "%s: setsockopt(SO_SNDTIMEO): %m",
+		       "upnp_event_create_notify");
+	}
+#endif
+	/* set socket non blocking */
 	if(!set_non_blocking(obj->s)) {
 		syslog(LOG_ERR, "%s: set_non_blocking(): %m",
 		       "upnp_event_create_notify");
@@ -233,9 +255,12 @@ upnp_event_notify_connect(struct upnp_event_notify * obj)
 	unsigned short port;
 #ifdef ENABLE_IPV6
 	struct sockaddr_storage addr;
+	socklen_t addrlen;
 #else
 	struct sockaddr_in addr;
+	socklen_t addrlen;
 #endif
+
 	if(!obj)
 		return;
 	memset(&addr, 0, sizeof(addr));
@@ -283,23 +308,28 @@ upnp_event_notify_connect(struct upnp_event_notify * obj)
 		sa->sin6_family = AF_INET6;
 		inet_pton(AF_INET6, obj->addrstr, &(sa->sin6_addr));
 		sa->sin6_port = htons(port);
+		addrlen = sizeof(struct sockaddr_in6);
 	} else {
 		struct sockaddr_in * sa = (struct sockaddr_in *)&addr;
 		sa->sin_family = AF_INET;
 		inet_pton(AF_INET, obj->addrstr, &(sa->sin_addr));
 		sa->sin_port = htons(port);
+		addrlen = sizeof(struct sockaddr_in);
 	}
 #else
 	addr.sin_family = AF_INET;
 	inet_aton(obj->addrstr, &addr.sin_addr);
 	addr.sin_port = htons(port);
+	addrlen = sizeof(struct sockaddr_in);
 #endif
 	syslog(LOG_DEBUG, "%s: '%s' %hu '%s'", "upnp_event_notify_connect",
 	       obj->addrstr, port, obj->path);
 	obj->state = EConnecting;
-	if(connect(obj->s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
+	if(connect(obj->s, (struct sockaddr *)&addr, addrlen) < 0) {
 		if(errno != EINPROGRESS && errno != EWOULDBLOCK) {
-			syslog(LOG_ERR, "%s: connect(): %m", "upnp_event_notify_connect");
+			syslog(LOG_ERR, "%s: connect(%d, %s, %u): %m",
+			       "upnp_event_notify_connect", obj->s,
+			       obj->addrstr, addrlen);
 			obj->state = EError;
 		}
 	}
@@ -377,7 +407,7 @@ static void upnp_event_send(struct upnp_event_notify * obj)
 {
 	int i;
 
-	syslog(LOG_DEBUG, "%s: sending event notify message to %s:%s",
+	syslog(LOG_DEBUG, "%s: sending event notify message to %s%s",
 	       "upnp_event_send", obj->addrstr, obj->portstr);
 	syslog(LOG_DEBUG, "%s: msg: %s",
 	       "upnp_event_send", obj->buffer + obj->sent);
@@ -439,7 +469,9 @@ upnp_event_process_notify(struct upnp_event_notify * obj)
 		}
 		if(err != 0) {
 			errno = err;
-			syslog(LOG_WARNING, "%s: connect failed: %m", "upnp_event_process_notify");
+			syslog(LOG_WARNING, "%s: connect(%s%s): %m",
+			       "upnp_event_process_notify",
+			       obj->addrstr, obj->portstr);
 			obj->state = EError;
 			break;
 		}
@@ -535,6 +567,8 @@ void upnpevents_processfds(fd_set *readset, fd_set *writeset)
 	for(sub = subscriberlist.lh_first; sub != NULL; ) {
 		subnext = sub->entries.le_next;
 		if(sub->timeout && curtime > sub->timeout && sub->notify == NULL) {
+			syslog(LOG_INFO, "subscriber timeouted : %u > %u SID=%s",
+			       (unsigned)curtime, (unsigned)sub->timeout, sub->uuid);
 			LIST_REMOVE(sub, entries);
 			free(sub);
 		}
