@@ -1,7 +1,7 @@
-/* $Id: upnpglobalvars.h,v 1.35 2013/06/13 13:21:30 nanard Exp $ */
+/* $Id: upnpglobalvars.h,v 1.41 2014/05/22 07:51:08 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2012 Thomas Bernard
+ * (c) 2006-2014 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -32,6 +32,9 @@ extern unsigned long upstream_bitrate;
 /* statup time */
 extern time_t startup_time;
 
+extern unsigned long int min_lifetime;
+extern unsigned long int max_lifetime;
+
 /* runtime boolean flags */
 extern int runtime_flags;
 #define LOGPACKETSMASK		0x0001
@@ -47,6 +50,16 @@ extern int runtime_flags;
 #ifdef PF_ENABLE_FILTER_RULES
 #define PFNOQUICKRULESMASK	0x0040
 #endif
+#ifdef ENABLE_IPV6
+#define IPV6DISABLEDMASK	0x0080
+#endif
+#ifdef ENABLE_6FC_SERVICE
+#define IPV6FCFWDISABLEDMASK		0x0100
+#define IPV6FCINBOUNDDISALLOWEDMASK	0x0200
+#endif
+#ifdef ENABLE_PCP
+#define PCP_ALLOWTHIRDPARTYMASK	0x0400
+#endif
 
 #define SETFLAG(mask)	runtime_flags |= mask
 #define GETFLAG(mask)	(runtime_flags & mask)
@@ -67,20 +80,33 @@ extern char modelnumber[];
 #define PRESENTATIONURL_MAX_LEN (64)
 extern char presentationurl[];
 
+#ifdef ENABLE_MANUFACTURER_INFO_CONFIGURATION
 #define FRIENDLY_NAME_MAX_LEN (64)
 extern char friendly_name[];
 
+#define MANUFACTURER_NAME_MAX_LEN (64)
+extern char manufacturer_name[];
+
+#define MANUFACTURER_URL_MAX_LEN (64)
+extern char manufacturer_url[];
+
+#define MODEL_NAME_MAX_LEN (64)
+extern char model_name[];
+
+#define MODEL_DESCRIPTION_MAX_LEN (64)
+extern char model_description[];
+
+#define MODEL_URL_MAX_LEN (64)
+extern char model_url[];
+#endif
+
 /* UPnP permission rules : */
 extern struct upnpperm * upnppermlist;
 extern unsigned int num_upnpperm;
 
-#ifdef ENABLE_NATPMP
-/* NAT-PMP */
-#if 0
-extern unsigned int nextnatpmptoclean_timestamp;
-extern unsigned short nextnatpmptoclean_eport;
-extern unsigned short nextnatpmptoclean_proto;
-#endif
+#ifdef PCP_SADSCP
+extern struct dscp_values* dscp_values_list;
+extern unsigned int num_dscp_values;
 #endif
 
 /* For automatic removal of expired rules (with LeaseDuration) */
@@ -95,8 +121,9 @@ extern const char * tag;
 
 #ifdef USE_NETFILTER
 extern const char * miniupnpd_nat_chain;
+extern const char * miniupnpd_peer_chain;
 extern const char * miniupnpd_forward_chain;
-#ifdef ENABLE_6FC_SERVICE
+#ifdef ENABLE_UPNPPINHOLE
 extern const char * miniupnpd_v6_filter_chain;
 #endif
 #endif
@@ -113,6 +140,10 @@ extern struct lan_addr_list lan_addrs;
 #ifdef ENABLE_IPV6
 /* ipv6 address used for HTTP */
 extern char ipv6_addr_for_http_with_brackets[64];
+
+/* address used to bind local services */
+extern struct in6_addr ipv6_bind_addr;
+
 #endif
 
 extern const char * minissdpdsocketpath;
@@ -121,10 +152,5 @@ extern const char * minissdpdsocketpath;
 extern unsigned int upnp_bootid;
 extern unsigned int upnp_configid;
 
-#ifdef ENABLE_6FC_SERVICE
-extern int ipv6fc_firewall_enabled;
-extern int ipv6fc_inbound_pinhole_allowed;
-#endif
-
 #endif
 
