@@ -1,4 +1,4 @@
-/* $Id: upnppinhole.h,v 1.3 2012/09/27 15:47:15 nanard Exp $ */
+/* $Id: upnppinhole.h,v 1.5 2014/05/15 10:15:05 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2012 Thomas Bernard
@@ -10,7 +10,8 @@
 
 #include "config.h"
 
-#ifdef ENABLE_6FC_SERVICE
+#ifdef ENABLE_UPNPPINHOLE
+
 /* functions to be used by WANIPv6_FirewallControl implementation */
 
 #if 0
@@ -26,8 +27,9 @@ upnp_check_outbound_pinhole(int proto, int * timeout);
  * .. = error */
 int
 upnp_add_inboundpinhole(const char * raddr, unsigned short rport,
-              const char * iaddr, unsigned short iport,
-              int proto, unsigned int leasetime, int * uid);
+                        const char * iaddr, unsigned short iport,
+                        int proto, char * desc,
+                        unsigned int leasetime, int * uid);
 
 /*
  * return values :
@@ -39,10 +41,18 @@ upnp_get_pinhole_info(unsigned short uid,
                       unsigned short * rport,
                       char * iaddr, int iaddrlen,
                       unsigned short * iport,
-                      int * proto,
+                      int * proto, char * desc, int desclen,
                       unsigned int * leasetime,
                       unsigned int * packets);
 
+/*
+ * return values:
+ * -1 = not found
+ * 0 .. 65535 = uid of the rule for the index
+ */
+int
+upnp_get_pinhole_uid_by_index(int index);
+
 /* update the lease time */
 int
 upnp_update_inboundpinhole(unsigned short uid, unsigned int leasetime);
@@ -63,8 +73,6 @@ upnp_check_pinhole_working(const char * uid, char * eaddr, char * iaddr, unsigne
 int
 upnp_clean_expired_pinholes(unsigned int * next_timestamp);
 
-#endif /* ENABLE_6FC_SERVICE */
-
-#endif
-
+#endif /* ENABLE_UPNPPINHOLE */
 
+#endif /* !UPNPPINHOLE_H_INCLUDED */
