@@ -1,7 +1,7 @@
-/* $Id: upnpsoap.c,v 1.118 2013/06/13 13:21:30 nanard Exp $ */
+/* $Id: upnpsoap.c,v 1.127 2014/05/15 10:15:04 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2013 Thomas Bernard
+ * (c) 2006-2014 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -410,6 +410,14 @@ AddPortMapping(struct upnphttp * h, const char * action)
 	       action, eport, int_ip, iport, protocol, desc, leaseduration,
 	       r_host ? r_host : "NULL");
 
+	/* TODO : be compliant with IGD spec for updating existing port mappings.
+	See "WANIPConnection:1 Service Template Version 1.01" 2.2.20.PortMappingDescription :
+	Overwriting Previous / Existing Port Mappings:
+	If the RemoteHost, ExternalPort, PortMappingProtocol and InternalClient are
+	exactly the same as an existing mapping, the existing mapping values for InternalPort,
+	PortMappingDescription, PortMappingEnabled and PortMappingLeaseDuration are
+	overwritten.
+	*/
 	r = upnp_redirect(r_host, eport, int_ip, iport, protocol, desc, leaseduration);
 
 	ClearNameValueList(&data);
@@ -764,12 +772,23 @@ DeletePortMappingRange(struct upnphttp * h, const char * action)
 		return;
 	}
 
+	syslog(LOG_INFO, "%s: deleting external ports: %hu-%hu, protocol: %s",
+	       action, startport, endport, protocol);
+
 	port_list = upnp_get_portmappings_in_range(startport, endport,
 	                                           protocol, &number);
+	if(number == 0)
+	{
+		SoapError(h, 730, "PortMappingNotFound");
+		ClearNameValueList(&data);
+		return;
+	}
+
 	for(i = 0; i < number; i++)
 	{
 		r = upnp_delete_redirection(port_list[i], protocol);
-		/* TODO : check return value for errors */
+		syslog(LOG_INFO, "%s: deleting external port: %hu, protocol: %s: %s",
+		       action, port_list[i], protocol, r < 0 ? "failed" : "ok");
 	}
 	free(port_list);
 	BuildSendAndCloseSoapResp(h, resp, sizeof(resp)-1);
@@ -1212,19 +1231,21 @@ GetFirewallStatus(struct upnphttp * h, const char * action)
 
 	bodylen = snprintf(body, sizeof(body), resp,
 		action, "urn:schemas-upnp-org:service:WANIPv6FirewallControl:1",
-		ipv6fc_firewall_enabled, ipv6fc_inbound_pinhole_allowed, action);
+	    GETFLAG(IPV6FCFWDISABLEDMASK) ? 0 : 1,
+	    GETFLAG(IPV6FCINBOUNDDISALLOWEDMASK) ? 0 : 1,
+	    action);
 	BuildSendAndCloseSoapResp(h, body, bodylen);
 }
 
 static int
 CheckStatus(struct upnphttp * h)
 {
-	if (!ipv6fc_firewall_enabled)
+	if (GETFLAG(IPV6FCFWDISABLEDMASK))
 	{
 		SoapError(h, 702, "FirewallDisabled");
 		return 0;
 	}
-	else if(!ipv6fc_inbound_pinhole_allowed)
+	else if(GETFLAG(IPV6FCINBOUNDDISALLOWEDMASK))
 	{
 		SoapError(h, 703, "InboundPinholeNotAllowed");
 		return 0;
@@ -1484,7 +1505,7 @@ AddPinhole(struct upnphttp * h, const char * action)
 	 * InternalClient and Protocol are the same than an existing pinhole,
 	 * but LeaseTime is different, the device MUST extend the existing
 	 * pinhole's lease time and return the UniqueID of the existing pinhole. */
-	r = upnp_add_inboundpinhole(rem_host, rport, int_ip, iport, proto, ltime, &uid);
+	r = upnp_add_inboundpinhole(rem_host, rport, int_ip, iport, proto, "IGD2 pinhole", ltime, &uid);
 
 	switch(r)
 	{
@@ -1549,7 +1570,9 @@ UpdatePinhole(struct upnphttp * h, const char * action)
 	 * it doesn't have access to, because of its public access */
 	n = upnp_get_pinhole_info(uid, NULL, 0, NULL,
 	                          iaddr, sizeof(iaddr), &iport,
-	                          NULL, NULL, NULL);
+	                          NULL, /* proto */
+	                          NULL, 0, /* desc, desclen */
+	                          NULL, NULL);
 	if (n >= 0)
 	{
 		if(PinholeVerification(h, iaddr, iport) <= 0)
@@ -1596,7 +1619,7 @@ GetOutboundPinholeTimeout(struct upnphttp * h, const char * action)
 	int opt=0, proto=0;
 	unsigned short iport, rport;
 
-	if (!ipv6fc_firewall_enabled)
+	if (GETFLAG(IPV6FCFWDISABLEDMASK))
 	{
 		SoapError(h, 702, "FirewallDisabled");
 		return;
@@ -1671,7 +1694,9 @@ DeletePinhole(struct upnphttp * h, const char * action)
 	 * it doesn't have access to, because of its public access */
 	n = upnp_get_pinhole_info(uid, NULL, 0, NULL,
 	                          iaddr, sizeof(iaddr), &iport,
-	                          &proto, &leasetime, NULL);
+	                          &proto,
+	                          NULL, 0, /* desc, desclen */
+	                          &leasetime, NULL);
 	if (n >= 0)
 	{
 		if(PinholeVerification(h, iaddr, iport) <= 0)
@@ -1738,7 +1763,9 @@ CheckPinholeWorking(struct upnphttp * h, const char * action)
 	r = upnp_get_pinhole_info(uid,
 	                          NULL, 0, NULL,
 	                          iaddr, sizeof(iaddr), &iport,
-	                          NULL, NULL, &packets);
+	                          NULL, /* proto */
+	                          NULL, 0, /* desc, desclen */
+	                          NULL, &packets);
 	if (r >= 0)
 	{
 		if(PinholeVerification(h, iaddr, iport) <= 0)
@@ -1797,7 +1824,9 @@ GetPinholePackets(struct upnphttp * h, const char * action)
 	 * it doesn't have access to, because of its public access */
 	n = upnp_get_pinhole_info(uid, NULL, 0, NULL,
 	                          iaddr, sizeof(iaddr), &iport,
-	                          &proto, &leasetime, &packets);
+	                          &proto,
+	                          NULL, 0, /* desc, desclen */
+	                          &leasetime, &packets);
 	if (n >= 0)
 	{
 		if(PinholeVerification(h, iaddr, iport)<=0)
@@ -1817,6 +1846,104 @@ GetPinholePackets(struct upnphttp * h, const char * action)
 }
 #endif
 
+#ifdef ENABLE_DP_SERVICE
+static void
+SendSetupMessage(struct upnphttp * h, const char * action)
+{
+	static const char resp[] =
+		"<u:%sResponse "
+		"xmlns:u=\"%s\">"
+		"<NewOutMessage>%s</NewOutMessage>"
+		"</u:%sResponse>";
+	char body[1024];
+	int bodylen;
+	struct NameValueParserData data;
+	const char * ProtocolType;	/* string */
+	const char * InMessage;		/* base64 */
+	const char * OutMessage = "";	/* base64 */
+
+	ParseNameValue(h->req_buf + h->req_contentoff, h->req_contentlen, &data);
+	ProtocolType = GetValueFromNameValueList(&data, "NewProtocolType");	/* string */
+	InMessage = GetValueFromNameValueList(&data, "NewInMessage");	/* base64 */
+
+	if(ProtocolType == NULL || InMessage == NULL)
+	{
+		ClearNameValueList(&data);
+		SoapError(h, 402, "Invalid Args");
+		return;
+	}
+	/*if(strcmp(ProtocolType, "DeviceProtection:1") != 0)*/
+	if(strcmp(ProtocolType, "WPS") != 0)
+	{
+		ClearNameValueList(&data);
+		SoapError(h, 600, "Argument Value Invalid"); /* 703 ? */
+		return;
+	}
+	/* TODO : put here code for WPS */
+
+	bodylen = snprintf(body, sizeof(body), resp,
+	                   action, "urn:schemas-upnp-org:service:DeviceProtection:1",
+	                   OutMessage, action);
+	BuildSendAndCloseSoapResp(h, body, bodylen);
+	ClearNameValueList(&data);
+}
+
+static void
+GetSupportedProtocols(struct upnphttp * h, const char * action)
+{
+	static const char resp[] =
+		"<u:%sResponse "
+		"xmlns:u=\"%s\">"
+		"<NewProtocolList>%s</NewProtocolList>"
+		"</u:%sResponse>";
+	char body[1024];
+	int bodylen;
+	const char * ProtocolList =
+		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+		"<SupportedProtocols xmlns=\"urn:schemas-upnp-org:gw:DeviceProtection\""
+		" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
+		" xsi:schemaLocation=\"urn:schemas-upnp-org:gw:DeviceProtection"
+		" http://www.upnp.org/schemas/gw/DeviceProtection-v1.xsd\">"
+		"<Introduction><Name>WPS</Name></Introduction>"
+		"<Login><Name>PKCS5</Name></Login>"
+		"</SupportedProtocols>";
+
+	bodylen = snprintf(body, sizeof(body), resp,
+	                   action, "urn:schemas-upnp-org:service:DeviceProtection:1",
+	                   ProtocolList, action);
+	BuildSendAndCloseSoapResp(h, body, bodylen);
+}
+
+static void
+GetAssignedRoles(struct upnphttp * h, const char * action)
+{
+	static const char resp[] =
+		"<u:%sResponse "
+		"xmlns:u=\"%s\">"
+		"<NewRoleList>%s</NewRoleList>"
+		"</u:%sResponse>";
+	char body[1024];
+	int bodylen;
+	const char * RoleList = "Public"; /* list of roles separated by spaces */
+
+#ifdef ENABLE_HTTPS
+	if(h->ssl != NULL) {
+		/* we should get the Roles of the session (based on client certificate) */
+		X509 * peercert;
+		peercert = SSL_get_peer_certificate(h->ssl);
+		if(peercert != NULL) {
+			RoleList = "Admin Basic";
+			X509_free(peercert);
+		}
+	}
+#endif
+
+	bodylen = snprintf(body, sizeof(body), resp,
+	                   action, "urn:schemas-upnp-org:service:DeviceProtection:1",
+	                   RoleList, action);
+	BuildSendAndCloseSoapResp(h, body, bodylen);
+}
+#endif
 
 /* Windows XP as client send the following requests :
  * GetConnectionTypeInfo
@@ -1874,6 +2001,12 @@ soapMethods[] =
 	{ "DeletePinhole", DeletePinhole},			/* Required */
 	{ "CheckPinholeWorking", CheckPinholeWorking},	/* Optional */
 	{ "GetPinholePackets", GetPinholePackets},	/* Required */
+#endif
+#ifdef ENABLE_DP_SERVICE
+	/* DeviceProtection */
+	{ "SendSetupMessage", SendSetupMessage},	/* Required */
+	{ "GetSupportedProtocols", GetSupportedProtocols},	/* Required */
+	{ "GetAssignedRoles", GetAssignedRoles},	/* Required */
 #endif
 	{ 0, 0 }
 };
@@ -1900,8 +2033,12 @@ ExecuteSoapAction(struct upnphttp * h, const char * action, int n)
 		while(soapMethods[i].methodName)
 		{
 			len = strlen(soapMethods[i].methodName);
-			if(strncmp(p, soapMethods[i].methodName, len) == 0)
+			if((len == methodlen) && memcmp(p, soapMethods[i].methodName, len) == 0)
 			{
+#ifdef DEBUG
+				syslog(LOG_DEBUG, "Remote Call of SoapMethod '%s'\n",
+				       soapMethods[i].methodName);
+#endif
 				soapMethods[i].methodImpl(h, soapMethods[i].methodName);
 				return;
 			}
