@@ -1,4 +1,4 @@
-/* $Id: upnputils.h,v 1.4 2013/02/06 10:50:04 nanard Exp $ */
+/* $Id: upnputils.h,v 1.6 2014/03/31 12:32:57 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2011-2013 Thomas Bernard
@@ -29,5 +29,30 @@ set_non_blocking(int fd);
 struct lan_addr_s *
 get_lan_for_peer(const struct sockaddr * peer);
 
+
+/**
+ * define portability macros
+ */
+#if defined(__sun)
+static size_t _sa_len(const struct sockaddr *addr)
+{
+        if (addr->sa_family == AF_INET)
+                return (sizeof(struct sockaddr_in));
+        else if (addr->sa_family == AF_INET6)
+                return (sizeof(struct sockaddr_in6));
+        else
+                return (sizeof(struct sockaddr));
+}
+# define SA_LEN(sa) (_sa_len(sa))
+#else
+#if !defined(SA_LEN)
+# define SA_LEN(sa) ((sa)->sa_len)
+#endif
+#endif
+
+#ifndef MAX
+# define MAX(a,b) (((a)>(b))?(a):(b))
+#endif
+
 #endif
 
